
#include <stdio.h>
#include "stm32f4xx_dma.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_spi.h"
#include "misc.h"

//-------------------------------------------------------------------------------------
/* Definition for SPIx resources */
//-------------------------------------------------------------------------------------

#define  SPI                       SPI2
#define  SPI_CLK                   RCC_APB1Periph_SPI2
#define  SPI_CLK_INIT              RCC_APB1PeriphClockCmd


#define  SPI_SCK_PIN               GPIO_Pin_13                  /* PA.05 */
#define  SPI_SCK_GPIO_PORT         GPIOB                       /* GPIOA */
#define  SPI_SCK_GPIO_CLK          RCC_AHB1Periph_GPIOB
#define  SPI_SCK_SOURCE            GPIO_PinSource13
#define  SPI_SCK_AF                GPIO_AF_SPI2


#define  SPI_MOSI_PIN              GPIO_Pin_15                  /* PA.7 */
#define  SPI_MOSI_GPIO_PORT        GPIOB                       /* GPIOA */
#define  SPI_MOSI_GPIO_CLK         RCC_AHB1Periph_GPIOB
#define  SPI_MOSI_SOURCE           GPIO_PinSource15
#define  SPI_MOSI_AF               GPIO_AF_SPI2

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------
/* Definition for DMAx resources */
//-------------------------------------------------------------------------------------
#define SPI_PORT_DR_ADDRESS                SPI->DR

#define SPI_PORT_DMA                       DMA1
#define SPI_PORT_DMAx_CLK                  RCC_AHB1Periph_DMA1

#define SPI_PORT_TX_DMA_CHANNEL            DMA_Channel_0
#define SPI_PORT_TX_DMA_STREAM             DMA1_Stream4
#define SPI_PORT_TX_DMA_FLAG_FEIF          DMA_FLAG_FEIF4
#define SPI_PORT_TX_DMA_FLAG_DMEIF         DMA_FLAG_DMEIF4
#define SPI_PORT_TX_DMA_FLAG_TEIF          DMA_FLAG_TEIF4
#define SPI_PORT_TX_DMA_FLAG_HTIF          DMA_FLAG_HTIF4
#define SPI_PORT_TX_DMA_FLAG_TCIF          DMA_FLAG_TCIF4

#define SPI_PORT_DMA_TX_IRQn               DMA1_Stream4_IRQn
#define SPI_PORT_DMA_TX_IRQHandler         DMA1_Stream4_IRQHandler
//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

uint8_t buffer[11] = {0xAA,0xAA,0xAA,0x00,0xAA,0x00,0xAA,0x00,0xAA,0x00,0xAA};


void DMA_INIT(void)
{

	DMA_InitTypeDef DMA_InitStructure;

	RCC_AHB1PeriphClockCmd(SPI_PORT_DMAx_CLK, ENABLE);

	DMA_DeInit(SPI_PORT_TX_DMA_STREAM);
	DMA_StructInit(&DMA_InitStructure);


	DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t) & (SPI_PORT_DR_ADDRESS);
	DMA_InitStructure.DMA_Channel = SPI_PORT_TX_DMA_CHANNEL;
	DMA_InitStructure.DMA_DIR = DMA_DIR_MemoryToPeripheral;
	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;

	DMA_InitStructure.DMA_Memory0BaseAddr = (uint32_t)&buffer;
	DMA_InitStructure.DMA_BufferSize =  1; //wird neu geschrieben in putBufferDMA()

	DMA_Init(SPI_PORT_TX_DMA_STREAM, &DMA_InitStructure);
	DMA_ITConfig(SPI_PORT_TX_DMA_STREAM, DMA_IT_TC, ENABLE);

}


void SPI_INIT(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  SPI_InitTypeDef  SPI_InitStructure;

  /* Enable the SPI periph */
   SPI_CLK_INIT( SPI_CLK, ENABLE);

  /* Enable SCK, MOSI and MISO GPIO clocks */
  RCC_AHB1PeriphClockCmd( SPI_SCK_GPIO_CLK |  SPI_MOSI_GPIO_CLK , ENABLE);

  GPIO_PinAFConfig( SPI_SCK_GPIO_PORT,  SPI_SCK_SOURCE,  SPI_SCK_AF);
  GPIO_PinAFConfig( SPI_MOSI_GPIO_PORT,  SPI_MOSI_SOURCE,  SPI_MOSI_AF);

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_DOWN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  /* SPI SCK pin configuration */
  GPIO_InitStructure.GPIO_Pin =  SPI_SCK_PIN;
  GPIO_Init( SPI_SCK_GPIO_PORT, &GPIO_InitStructure);

  /* SPI  MOSI pin configuration */
  GPIO_InitStructure.GPIO_Pin =   SPI_MOSI_PIN;
  GPIO_Init( SPI_MOSI_GPIO_PORT, &GPIO_InitStructure);

  /* SPI configuration -------------------------------------------------------*/
  SPI_I2S_DeInit( SPI);
  SPI_InitStructure.SPI_Direction = SPI_Direction_1Line_Tx;
  SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
  SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
  SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
  SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_256;
  SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
  SPI_InitStructure.SPI_CRCPolynomial = 0;
  SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
  SPI_Init( SPI, &SPI_InitStructure);

  /* Enable SPI1  */
  SPI_Cmd( SPI, ENABLE);
}


void INIT_NVIC(void)
{
	NVIC_InitTypeDef NVIC_InitStructure;
	// enable the interrupt in the NVIC
	NVIC_InitStructure.NVIC_IRQChannel = SPI_PORT_DMA_TX_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
	// Enable dma tx request.
	SPI_I2S_DMACmd( SPI, SPI_I2S_DMAReq_Tx, ENABLE);
}


void putBufferDMA (uint8_t len)
{
	SPI_PORT_TX_DMA_STREAM->NDTR = (uint32_t) len;
	DMA_Cmd (SPI_PORT_TX_DMA_STREAM, ENABLE);
}


int main(void)
{

	SPI_INIT();
	INIT_NVIC();
	DMA_INIT();
	putBufferDMA(11);

}
